<?php

include('Rule.php');

/**
* class Rules
* Gère l'ensemble des règles d'une machine de Turing
* @author Grégory Barbier & Adrian Gaudebert - adrian@gaudebert.fr
*/
class Rules
{
    public $initialState = '';
    public $finalState = '';
    public $whiteSpace = '';
    public $rules = array();
    public $valide = false;
    
    /**
    * LoadFromFile
    * Charge les règles depuis un fichier envoyé par l'utilisateur
    * Renvoie true si le chargement a réussi, false autrement
    * @param $file Fichier envoyé par l'utilisateur
    * @return bool
    */
    public function LoadFromFile($file)
    {
        $this->Reset();
        
        return false;
    }
    
    /**
    * LoadFromInput
    * Charge les règles depuis un tableau de tableaux (données issues du formulaire)
    * @param $input Tableau de tableaux contenant les données du formulaire
    * @return none
    */
    public function LoadFromInput($input, $initialState, $finalState, $whiteSpace)
    {
        $this->Reset();
        
        $this->initialState = $initialState;
        $this->finalState = $finalState;
        $this->whiteSpace = $whiteSpace;
        
        foreach ($input as $rules)
        {
            $this->rules[] = new Rule(
                $rules['current_state'], 
                $rules['current_input'], 
                $rules['next_state'], 
                $rules['next_input'], 
                $rules['next_direction']);
        }
    }
    
    /**
    * Reset
    * Réinitialise les données de la classe (état initial, état final, règles)
    * @param none
    * @return none
    */
    public function Reset()
    {
        $this->initialState = '';
        $this->finalState = '';
        $this->rules = array();
        $this->valide = false;
    }
    
    /**
    * Validate
    * Vérifie la validité des règles entrées
    * Renvoie true si
    *   * l'état initial est défini
    *   * l'état final est défini
    *   * la machine entrée est déterministe
    *   * l'état final est atteignable
    * Renvoie false autrement
    * @param None
    * @return bool
    */
    public function Validate()
    {
        $res = array(
            'result' => true, 
            'error' => ''
        );
        $all = array();
        
        // On vérifie si la machine est déterministe
        $sameRules = false;
        $rules = $this->rules;
        
        foreach ($this->rules as $rule1)
        {
            array_shift($rules);    // Retrait du premier élément de la liste
            
            foreach ($rules as $rule2)
            {
                if ($rule1->currentState == $rule2->currentState && 
                    $rule1->currentInput == $rule2->currentInput)
                {
                    $sameRules = true;
                    break;
                }
            }
            if ($sameRules)
            {
                break;
            }
        }
        
        if ($sameRules)
        {
            // Erreur, la machine est indéterministe
            $res['result'] = false;
            $res['error'] = 'La machine est indéterministe';
            return $res;
        }
        
        // Il existe un état final
        $isFinalState = false;
        foreach ($this->rules as $rule)
        {
            if ($rule->nextState == $this->finalState)
            {
                $isFinalState = true;
                break;
            }
        }
        if (!$isFinalState)
        {
            // erreur : pas d'état final
            $res['result'] = false;
            $res['error'] = 'L\'état final n\'est pas présent dans les règles';
            return $res;
        }
        
        // Formatage des données pour faciliter leur exploitation (création d'un arbre)
        foreach ($this->rules as $rule)
        {
            if ($rule->currentState != $rule->nextState)
            {
                $state = $rule->currentState;
                
                if (!array_key_exists($state, $all))
                {
                    $all[$state] = array();
                }
                
                $all[$state][] = $rule;
            }
        }
        
        // Il existe un état initial
        if (!isSet($all[$this->initialState]))
        {
            // erreur : pas d'état initial
            $res['result'] = false;
            $res['error'] = 'L\'état initial n\'est pas présent dans les règles';
            return $res;
        }
        
        $state = $this->initialState;
        $done = array();
        
        // On vérifie que l'état final peut bien être atteint avec les règles de transition
        if (!$this->Explore($all, $state, $done))
        {
            // Erreur : état final non atteignable
            $res['result'] = false;
            $res['error'] = 'L\'état final ne peut pas être atteint par la machine';
            return $res;
        }
        
        // La fonction n'a pas pas été interrompue, donc les règles sont valides
        $this->valide = true;
        
        return $res;
    }
    
    /**
    * Explore
    * Fonction récursive qui explore les règles jusqu'à atteindre l'état final
    * Renvoie true si l'état final peut être atteint, false sinon
    * @param $all Le tableau contenant les règles
    * @param $state L'état courant
    * @param $done La liste des états déjà visités
    * @return bool
    */
    private function Explore($all, $state, $done)
    {
        // On classe l'état courant comme étant déjà traité
        $done[] = $state;
        
        if (array_key_exists($state, $all))
        {
            // Pour chaque règle correponsdant à l'état courant
            foreach ($all[$state] as $rule)
            {
                // Si l'état suivant est l'état final, on arrête et on revoie true
                if ($rule->nextState == $this->finalState)
                {
                    return true;
                }
                // Si l'état suivant est déjà vu, on ne fait rien
                else if (in_array($rule->nextState, $done))
                {
                }
                // Sinon, on explore les règles correpondantes à l'état suivant
                else
                {
                    if ($this->Explore($all, $rule->nextState, $done))
                    {
                        return true;
                    }
                }
            }
        }
        
        return false;
    }
}

?>
